#include <zephyr/kernel.h>
#include <zephyr/device.h>
#include <zephyr/storage/flash_map.h>
#include <zephyr/fs/nvs.h>
#include <cJSON.h>
#include <string.h>
#include <zephyr/drivers/flash.h>

static struct nvs_fs fs;

#define NVS_PARTITION          storage_partition
#define NVS_PARTITION_DEVICE   FIXED_PARTITION_DEVICE(NVS_PARTITION)
#define NVS_PARTITION_OFFSET   FIXED_PARTITION_OFFSET(NVS_PARTITION)

#define JSON_DATA_ID 10
#define SLEEP_TIME   2000  // 2 seconds

void store_json_data(struct nvs_fs *fs) {
    cJSON *root = cJSON_CreateObject();

    // Add actual numeric values
    cJSON_AddStringToObject(root, "name", "BluetoothBeacon");

	 char tx_power_str[16]; 
    char battery_level_str[16];

    // Convert tx_power and battery_level to strings
    sprintf(tx_power_str, "%d", -40);  // Example TX power value as string
    sprintf(battery_level_str, "%d", 75);  // Example battery level value as string

    // Add the strings to JSON
    cJSON_AddStringToObject(root, "tx_power", tx_power_str);
    cJSON_AddStringToObject(root, "battery_level", battery_level_str);


    //cJSON_AddNumberToObject(root, "tx_power", -40);  // Example TX power value
    //cJSON_AddNumberToObject(root, "battery_level", 75);  // Example battery level value

    const char *json_string = cJSON_PrintUnformatted(root);
    printk("JSON data stored: %s\n", json_string);

    // Write to NVS
    int rc = nvs_write(fs, JSON_DATA_ID, json_string, strlen(json_string) + 1);
    if (rc < 0) {
        printk("Error: Failed to write JSON data to NVS, rc=%d\n", rc);
    } else {
        printk("JSON data successfully written to NVS\n");
    }

    cJSON_Delete(root);
}

#include <stdlib.h>  // For strtol

void read_json_data(struct nvs_fs *fs) {
    char json_buffer[256]; 
    
    int rc = nvs_read(fs, JSON_DATA_ID, json_buffer, sizeof(json_buffer));
    
    if (rc > 0) {
        json_buffer[rc] = '\0';  // Null-terminate the string
        printk("JSON data read: %s\n", json_buffer);

        cJSON *root = cJSON_Parse(json_buffer);
        if (root != NULL) {
            const cJSON *name = cJSON_GetObjectItem(root, "name");
            const cJSON *tx_power_str = cJSON_GetObjectItem(root, "tx_power");
            const cJSON *battery_level_str = cJSON_GetObjectItem(root, "battery_level");

            if (name && tx_power_str && battery_level_str) {
                printk("Parsed JSON: name=%s, tx_power=%s, battery_level=%s\n",
                       name->valuestring,  
                       tx_power_str->valuestring, 
                       battery_level_str->valuestring);

                // Convert tx_power and battery_level to integers using strtol
                char *endptr;
                long tx_power = strtol(tx_power_str->valuestring, &endptr, 10);  // Convert to integer
                long battery_level = strtol(battery_level_str->valuestring, &endptr, 10);  // Convert to integer

                printk("Converted values: tx_power=%ld, battery_level=%ld\n", tx_power, battery_level);
            } else {
                printk("Error: JSON parsing error - missing fields\n");
            }

            cJSON_Delete(root);
        } else {
            printk("Error: Failed to parse JSON data\n");
        }
    } else if (rc == 0) {
        printk("Warning: No JSON data found in NVS\n");
    } else {
        printk("Error: Failed to read JSON data, rc=%d\n", rc);
    }
}

int main(void) {


    int rc;
    struct flash_pages_info info;

    // Initialize NVS file system
    fs.flash_device = NVS_PARTITION_DEVICE;
    if (!device_is_ready(fs.flash_device)) {
        printk("Error: Flash device not ready\n");
        return -1;
    }
   

    fs.offset = NVS_PARTITION_OFFSET;
    rc = flash_get_page_info_by_offs(fs.flash_device, fs.offset, &info);
    if (rc) {
        printk("Error: Failed to get page info, rc=%d\n", rc);
        return -1;
    }

    fs.sector_size = info.size;
    fs.sector_count = 3U;

    rc = nvs_mount(&fs);
    if (rc) {
        printk("Error: Failed to mount NVS, rc=%d\n", rc);
        return -1;
    }

    // Store and read JSON data
    store_json_data(&fs);
    read_json_data(&fs);

    while (1) {
        k_msleep(SLEEP_TIME);
    }

    return 0;
}
